#ifndef MappingResults_H
#define MappingResults_H

#include <vector>
#include <string>
#include <fstream>
#include <map>
using namespace std;

struct enrichedPair
{
   string name;
   double pValue;
   double adjustedPValue;
   double ADVValue;
   double LogLikelihood;
   double UncertaintyCoefficient;
};

class MappingResults
{
   public:
      MappingResults(string, vector<string>&);
      MappingResults(string mName) {name=mName;}
      MappingResults(char*);
      ~MappingResults();
      int getMappedSize() {return mapped.size();}
      vector<string>& getMapped() {return mapped;}
      int getCommonEleCount(vector<string> &);
      void uniqueAndOrder();
      void addElement(string);
      void printFasta(ostream &);
      string getName() {return name;}
      void addEnriched(string termName, double pValue);
      void addEnriched(string termName, double pValue, double logLikelihood, double UncertaintyCoefficient);
      void addEnrichedWithAdjPValue(string termName, double pValue, double adjPValue);
      void printEnriched(ostream &, double);
      void calculateAdjustedPValue();
      vector<enrichedPair>& getEnriched() {return enriched;}
      void calculateSummaryScore(MappingResults &);
      void calculateSummaryScore(MappingResults &, double, int);
      void calculateSummaryScore(MappingResults &, double);
      void calculateSummaryScoreForGene(MappingResults &, double);
      void calculateSummaryScoreForGeneNoPenalty(MappingResults &, double);
      void printSummaryScore(ostream &);
      void printSummaryScore(ostream &, int);
      void printTopOnto(ostream &);
      void printTopOnto(ostream &, string);
      void printTopOnto(ostream &, string, string);
      void calculateTopOntoPairs(MappingResults &, string, double, int);
      void emptyTopOnto() {topOnto.clear();}
      void emptySummaryScoreWCF() {summaryScoreWithCutOff.clear();}
   private:
      string name;
      vector<string> mapped;
      vector<enrichedPair> enriched;
      map<string, double> summaryScore;
      map<string, map<double, double> > summaryScoreWithCutOff;
      map<string, vector<pair<string, double> > > topOnto; 
      double calculateSMS(vector<enrichedPair> &);
      double calculateSMS(vector<enrichedPair> &, double);
      double calculateSMSForGene(vector<enrichedPair> &, double);
      double calculateSMSForGeneNoPenalty(vector<enrichedPair> &, double);
      void getTopOntoPairs(vector<pair<string, double> >&, string, int, string);
};

#endif
