#include <fstream>
#include <string>
#include <iostream>
#include <iomanip>
#include <map>
#include <algorithm>
#include <vector>
#include <cstdlib>
#include <sstream>
using namespace std;

#include "gsl_randist.h"
#include "gsl_cdf.h"
#include "MappingResults.h"
#include "CalculateWeight.h"

//compile command (modify to use the rigth header file and library path): g++ MappingResults.cpp makeOntoFin.cpp -o makeOntoFin -I/usr/include/gsl -L/lib64 -lgsl -lgslcblas

int main(int argc, char* argv[])
{
   
   if (argc != 6) //Usage description
   {
      cout<<"Usage: "<<argv[0]<<" geneMappingFile ontoMappingFile output p-valueCutOff numbOfTotalGeneLinkedAbs"<<endl;
      exit(0);
   }

   int TotalAbs=atoi(argv[5]);
   double cutoff = atof(argv[4]);
   ifstream onto(argv[2]);
   vector<MappingResults> ontoMap;
   string line, s;
   getline(onto, line, '>');
   while (getline(onto, line, '>')) //Read in gene2PubMed file
   {
     istringstream is(line);
     getline(is, s);
     MappingResults mp(s);
     while(getline(is, s))
        if (!s.empty())
           mp.addElement(s);

     mp.uniqueAndOrder();
     ontoMap.push_back(mp);
   }

   int m, n, k, x;
   vector<MappingResults>::iterator oItr;
   ofstream out(argv[3]);

   ifstream geneIn(argv[1]);
   getline(geneIn, s, '>');
   while (getline(geneIn, line, '>'))  //Read in ontology to PubMed file
   {
     istringstream is(line);
     getline(is, s);
     MappingResults mp(s);
     while(getline(is, s))
        if (!s.empty())
           mp.addElement(s);

     mp.uniqueAndOrder();

     m=mp.getMappedSize();
     n=TotalAbs - m;

     for (oItr=ontoMap.begin(); oItr!=ontoMap.end(); ++oItr) //make Ontology Fingerprint for each gene
     {
        x = oItr->getCommonEleCount(mp.getMapped());
        if (x > 0)
        {
           k = oItr->getMappedSize();
           double test = gsl_ran_hypergeometric_pdf(x,m,n,k);
           double test2 = gsl_cdf_hypergeometric_P(x,m,n,k);
           double d=(1-test2)+(test*.5);
           if (d<1)
             mp.addEnriched(oItr->getName(), d);
        }
     }

     mp.calculateAdjustedPValue();

     mp.printEnriched(out, cutoff);
   }

   return 0;
} 
